@@ -165,10 +165,10 @@ wnck_class_group_finalize (GObject *object)
  *
  * Gets the #WnckClassGroup corresponding to @res_class.
  *
- * Return value: the #WnckClassGroup corresponding to @res_class, or %NULL if
- * there is no #WnckClassGroup with the specified @res_class. The returned
- * #WnckClassGroup is owned by libwnck and must not be referenced or
- * unreferenced.
+ * Return value: (transfer none): the #WnckClassGroup corresponding to
+ * @res_class, or %NULL if there is no #WnckClassGroup with the specified
+ * @res_class. The returned #WnckClassGroup is owned by libwnck and must not be
+ * referenced or unreferenced.
  *
  * Since: 2.2
  **/
@@ -413,6 +413,7 @@ static void
 set_icon (WnckClassGroup *class_group)
 {
   GdkPixbuf *icon, *mini_icon;
+  gboolean icons_reffed = FALSE;
 
   get_icons_from_applications (class_group, &icon, &mini_icon);
 
@@ -420,12 +421,15 @@ set_icon (WnckClassGroup *class_group)
     get_icons_from_windows (class_group, &icon, &mini_icon);
 
   if (!icon || !mini_icon)
+    {
       _wnck_get_fallback_icons (&icon,
                                 DEFAULT_ICON_WIDTH,
                                 DEFAULT_ICON_HEIGHT,
                                 &mini_icon,
                                 DEFAULT_MINI_ICON_WIDTH,
                                 DEFAULT_MINI_ICON_HEIGHT);
+      icons_reffed = TRUE;
+    }
 
   g_assert (icon && mini_icon);
 
@@ -435,8 +439,14 @@ set_icon (WnckClassGroup *class_group)
   if (class_group->priv->mini_icon)
     g_object_unref (class_group->priv->mini_icon);
 
-  class_group->priv->icon = g_object_ref (icon);
-  class_group->priv->mini_icon = g_object_ref (mini_icon);
+  class_group->priv->icon = icon;
+  class_group->priv->mini_icon = mini_icon;
+
+  if (!icons_reffed)
+    {
+      g_object_ref (class_group->priv->icon);
+      g_object_ref (class_group->priv->mini_icon);
+    }
 
   g_signal_emit (G_OBJECT (class_group), signals[ICON_CHANGED], 0);
 }
@@ -474,7 +484,7 @@ _wnck_class_group_add_window (WnckClassGroup *class_group,
  * _wnck_class_group_remove_window:
  * @class_group: a #WnckClassGroup.
  * @window: a #WnckWindow.
- * 
+ *
  * Removes a window from the list of windows that are grouped under the
  * specified @class_group.
  **/
@@ -497,12 +507,13 @@ _wnck_class_group_remove_window (WnckClassGroup *class_group,
 /**
  * wnck_class_group_get_windows:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the list of #WnckWindow that are grouped in @class_group.
- * 
- * Return value: the list of #WnckWindow grouped in @class_group, or %NULL if
- * the group contains no window. The list should not be modified nor freed, as
- * it is owned by @class_group.
+ *
+ * Return value: (element-type WnckWindow) (transfer none): the list of
+ * #WnckWindow grouped in @class_group, or %NULL if the group contains no
+ * window. The list should not be modified nor freed, as it is owned by
+ * @class_group.
  *
  * Since: 2.2
  **/
@@ -517,9 +528,9 @@ wnck_class_group_get_windows (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_res_class:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the resource class name for @class_group.
- * 
+ *
  * Return value: the resource class name of @class_group, or an
  * empty string if the group has no resource class name.
  *
@@ -536,7 +547,7 @@ wnck_class_group_get_res_class (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_name:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets an human-readable name for @class_group. Since there is no way to
  * properly find this name, a suboptimal heuristic is used to find it. The name
  * is the name of all #WnckApplication for each #WnckWindow in @class_group if
@@ -544,7 +555,7 @@ wnck_class_group_get_res_class (WnckClassGroup *class_group)
  * name, the name is the name of all #WnckWindow in @class_group if they all
  * have the same name. If all #WnckWindow don't have the same name, the
  * resource class name is used.
- * 
+ *
  * Return value: an human-readable name for @class_group.
  *
  * Since: 2.2
@@ -560,16 +571,16 @@ wnck_class_group_get_name (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_icon:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the icon to be used for @class_group. Since there is no way to
  * properly find the icon, a suboptimal heuristic is used to find it. The icon
  * is the first icon found by looking at all the #WnckApplication for each
  * #WnckWindow in @class_group, then at all the #WnckWindow in @class_group. If
  * no icon was found, a fallback icon is used.
- * 
- * Return value: the icon for @class_group. The caller should reference the
- * returned <classname>GdkPixbuf</classname> if it needs to keep the icon
- * around.
+ *
+ * Return value: (transfer none): the icon for @class_group. The caller should
+ * reference the returned <classname>GdkPixbuf</classname> if it needs to keep
+ * the icon around.
  *
  * Since: 2.2
  **/
@@ -584,14 +595,14 @@ wnck_class_group_get_icon (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_mini_icon:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the mini-icon to be used for @class_group. Since there is no way to
  * properly find the mini-icon, the same suboptimal heuristic as the one for
  * wnck_class_group_get_icon() is used to find it.
- * 
- * Return value: the mini-icon for @class_group. The caller should reference
- * the returned <classname>GdkPixbuf</classname> if it needs to keep the
- * mini-icon around.
+ *
+ * Return value: (transfer none): the mini-icon for @class_group. The caller
+ * should reference the returned <classname>GdkPixbuf</classname> if it needs
+ * to keep the mini-icon around.
  *
  * Since: 2.2
  **/
