@@ -88,12 +88,12 @@ struct _WnckWindowPrivate
   gint sort_order;
 
   WnckWindowType wintype;
-  
+
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
 
   WnckIconCache *icon_cache;
-  
+
   WnckWindowActions actions;
 
   int x;
@@ -110,12 +110,12 @@ struct _WnckWindowPrivate
 
   char *res_class;
   char *res_name;
-  
+
   /* true if transient_for points to root window,
    * not another app window
    */
   guint transient_for_root : 1;
-  
+
   /* window state */
   guint is_minimized : 1;
   guint is_maximized_horz : 1;
@@ -137,9 +137,9 @@ struct _WnckWindowPrivate
    * externally-visible state (it determines the WM_STATE
    * interpretation)
    */
-  guint net_wm_state_hidden : 1;  
+  guint net_wm_state_hidden : 1;
   guint wm_state_iconic : 1;
-  
+
   /* idle handler for updates */
   guint update_handler;
 
@@ -324,7 +324,7 @@ wnck_window_class_init (WnckWindowClass *klass)
    * WnckWindow::state-changed:
    * @window: the #WnckWindow which emitted the signal.
    * @changed_mask: the bitmask containing bits set for all states of @window
-   * that have changed. 
+   * that have changed.
    * @new_state: the new state of @window.
    *
    * Emitted when the state of @window changes. This can happen when @window is
@@ -377,7 +377,7 @@ wnck_window_class_init (WnckWindowClass *klass)
    * WnckWindow::actions-changed:
    * @window: the #WnckWindow which emitted the signal.
    * @changed_mask: the bitmask containing bits set for all actions
-   * availabilities for @window that have changed. 
+   * availabilities for @window that have changed.
    * @new_state: the new actions availabilities for @window.
    *
    * Emitted when the actions availabilities for @window change.
@@ -406,7 +406,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_STRUCT_OFFSET (WnckWindowClass, geometry_changed),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);  
+                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -442,7 +442,7 @@ wnck_window_finalize (GObject *object)
   if (window->priv->mini_icon)
     g_object_unref (G_OBJECT (window->priv->mini_icon));
   window->priv->mini_icon = NULL;
-  
+
   _wnck_icon_cache_free (window->priv->icon_cache);
   window->priv->icon_cache = NULL;
 
@@ -464,8 +464,8 @@ wnck_window_finalize (GObject *object)
  * create a #WnckWindow if none exists. The function is robust against bogus
  * window IDs.
  *
- * Return value: the #WnckWindow for @xwindow. The returned #WnckWindow is
- * owned by libwnck and must not be referenced or unreferenced.
+ * Return value: (transfer none): the #WnckWindow for @xwindow. The returned
+ * #WnckWindow is owned by libwnck and must not be referenced or unreferenced.
  **/
 WnckWindow*
 wnck_window_get (gulong xwindow)
@@ -482,8 +482,8 @@ wnck_window_get (gulong xwindow)
  *
  * Gets the #WnckScreen @window is on.
  *
- * Return value: the #WnckScreen @window is on. The returned #WnckScreen is
- * owned by libwnck and must not be referenced or unreferenced.
+ * Return value: (transfer none): the #WnckScreen @window is on. The returned
+ * #WnckScreen is owned by libwnck and must not be referenced or unreferenced.
  **/
 WnckScreen*
 wnck_window_get_screen (WnckWindow *window)
@@ -499,6 +499,7 @@ _wnck_window_create (Window      xwindow,
                      gint        sort_order)
 {
   WnckWindow *window;
+  Screen     *xscreen;
 
   if (window_hash == NULL)
     window_hash = g_hash_table_new (_wnck_xid_hash, _wnck_xid_equal);
@@ -506,6 +507,8 @@ _wnck_window_create (Window      xwindow,
   g_return_val_if_fail (g_hash_table_lookup (window_hash, &xwindow) == NULL,
                         NULL);
 
+  xscreen = WNCK_SCREEN_XSCREEN (screen);
+
   window = g_object_new (WNCK_TYPE_WINDOW, NULL);
   window->priv->xwindow = xwindow;
   window->priv->screen = screen;
@@ -518,7 +521,8 @@ _wnck_window_create (Window      xwindow,
    * that's why we select the union of the mask we want for Application
    * and the one we want for window
    */
-  _wnck_select_input (window->priv->xwindow,
+  _wnck_select_input (xscreen,
+                      window->priv->xwindow,
                       WNCK_APP_WINDOW_EVENT_MASK);
 
   /* Default the group leader to the window itself; it is set in
@@ -527,16 +531,16 @@ _wnck_window_create (Window      xwindow,
   window->priv->group_leader = window->priv->xwindow;
 
   window->priv->session_id =
-    _wnck_get_session_id (window->priv->xwindow);
+    _wnck_get_session_id (xscreen, window->priv->xwindow);
 
   window->priv->pid =
-    _wnck_get_pid (window->priv->xwindow);
+    _wnck_get_pid (xscreen, window->priv->xwindow);
 
   window->priv->x = 0;
   window->priv->y = 0;
   window->priv->width = 0;
   window->priv->height = 0;
-  _wnck_get_window_geometry (WNCK_SCREEN_XSCREEN (window->priv->screen),
+  _wnck_get_window_geometry (xscreen,
 			     xwindow,
                              &window->priv->x,
                              &window->priv->y,
@@ -544,7 +548,7 @@ _wnck_window_create (Window      xwindow,
                              &window->priv->height);
 
   window->priv->sort_order = sort_order;
-  
+
   window->priv->need_update_name = TRUE;
   window->priv->need_update_state = TRUE;
   window->priv->need_update_icon_name = TRUE;
@@ -579,6 +583,12 @@ _wnck_window_destroy (WnckWindow *window)
   g_object_unref (G_OBJECT (window));
 }
 
+static Display *
+_wnck_window_get_display (WnckWindow *window)
+{
+  return DisplayOfScreen (WNCK_SCREEN_XSCREEN (window->priv->screen));
+}
+
 /**
  * wnck_window_has_name:
  * @window: a #WnckWindow.
@@ -692,12 +702,12 @@ _wnck_window_get_name_for_display (WnckWindow *window,
   const char *name;
 
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
-  
+
   if (use_icon_name && wnck_window_has_icon_name (window))
     name = wnck_window_get_icon_name (window);
-  else 
+  else
     name = wnck_window_get_name (window);
-  
+
   if (use_state_decorations)
     {
       if (window->priv->is_shaded)
@@ -718,9 +728,9 @@ _wnck_window_get_name_for_display (WnckWindow *window,
  *
  * Gets the #WnckApplication to which @window belongs.
  *
- * Return value: the #WnckApplication to which @window belongs. The returned
- * #WnckApplication is owned by libwnck and must not be referenced or
- * unreferenced.
+ * Return value: (transfer none): the #WnckApplication to which @window belongs.
+ * The returned #WnckApplication is owned by libwnck and must not be referenced
+ * or unreferenced.
  **/
 WnckApplication*
 wnck_window_get_application  (WnckWindow *window)
@@ -736,8 +746,10 @@ wnck_window_get_application  (WnckWindow *window)
  *
  * Gets the #WnckWindow for which @window is transient.
  *
- * Return value: the #WnckWindow for which @window is transient, or %NULL if
- * @window is not transient for any #WnckWindow.
+ * Return value: (transfer none): the #WnckWindow for which @window is
+ * transient, or %NULL if @window is not transient for any #WnckWindow.
+ * The returned #WnckWindow is owned by libwnck and must not be referenced or
+ * unreferenced.
  *
  * Since: 2.12
  **/
@@ -789,9 +801,9 @@ wnck_window_get_xid (WnckWindow *window)
  *
  * Gets the #WnckClassGroup to which @window belongs.
  *
- * Return value: the #WnckClassGroup to which @window belongs. The returned
- * #WnckClassGroup is owned by libwnck and must not be referenced or
- * unreferenced.
+ * Return value: (transfer none): the #WnckClassGroup to which @window belongs.
+ * The returned #WnckClassGroup is owned by libwnck and must not be referenced
+ * or unreferenced.
  *
  * Since: 2.2
  **/
@@ -907,11 +919,11 @@ wnck_window_get_sort_order (WnckWindow *window)
  *
  * Since: 2.20
  **/
-void        wnck_window_set_sort_order        (WnckWindow *window, 
+void        wnck_window_set_sort_order        (WnckWindow *window,
 					       gint order)
-{ 
+{
   g_return_if_fail (WNCK_IS_WINDOW (window));
-  
+
   window->priv->sort_order = order;
   return;
 }
@@ -919,9 +931,9 @@ void        wnck_window_set_sort_order        (WnckWindow *window,
 /**
  * wnck_window_get_window_type:
  * @window: a #WnckWindow.
- * 
+ *
  * Gets the semantic type of @window.
- * 
+ *
  * Return value: the semantic type of @window.
  **/
 WnckWindowType
@@ -929,7 +941,7 @@ wnck_window_get_window_type (WnckWindow *window)
 {
   /* FIXME: should we have an invalid window type for this? */
   g_return_val_if_fail (WNCK_IS_WINDOW (window), 0);
-  
+
   return window->priv->wintype;
 }
 
@@ -937,7 +949,7 @@ wnck_window_get_window_type (WnckWindow *window)
  * wnck_window_set_window_type:
  * @window: a #WnckWindow.
  * @wintype: a semantic type.
- * 
+ *
  * Sets the semantic type of @window to @wintype.
  *
  * Since: 2.12
@@ -946,6 +958,7 @@ void
 wnck_window_set_window_type (WnckWindow *window, WnckWindowType wintype)
 {
   Atom atom;
+  Display *display;
 
   g_return_if_fail (WNCK_IS_WINDOW (window));
 
@@ -972,20 +985,23 @@ wnck_window_set_window_type (WnckWindow *window, WnckWindowType wintype)
     atom = _wnck_atom_get ("_NET_WM_WINDOW_TYPE_UTILITY");
     break;
   case WNCK_WINDOW_SPLASHSCREEN:
-    atom = _wnck_atom_get ("_NET_WM_WINDOW_TYPE_SPLASHSCREEN");
+    atom = _wnck_atom_get ("_NET_WM_WINDOW_TYPE_SPLASH");
     break;
   default:
     return;
   }
-  _wnck_error_trap_push ();
 
-  XChangeProperty (gdk_display,
-                   window->priv->xwindow, 
+  display = _wnck_window_get_display (window);
+
+  _wnck_error_trap_push (display);
+
+  XChangeProperty (display,
+                   window->priv->xwindow,
                    _wnck_atom_get ("_NET_WM_WINDOW_TYPE"),
 		   XA_ATOM, 32, PropModeReplace,
 		   (guchar *)&atom, 1);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 /**
@@ -1041,7 +1057,7 @@ transient_needs_attention (WnckWindow *window)
 {
   GList *windows;
   WnckWindow *transient;
-  
+
   if (!WNCK_IS_WINDOW (window))
     return NULL;
 
@@ -1095,7 +1111,7 @@ _wnck_window_or_transient_get_needs_attention_time (WnckWindow *window)
 gboolean
 wnck_window_or_transient_needs_attention (WnckWindow *window)
 {
-  return wnck_window_needs_attention (window) || 
+  return wnck_window_needs_attention (window) ||
          transient_needs_attention (window) != NULL;
 }
 
@@ -1137,16 +1153,16 @@ const char*
 _wnck_window_get_startup_id (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
-  
+
   if (window->priv->startup_id == NULL &&
       window->priv->group_leader != None)
     {
       WnckApplication *app;
 
       /* Fall back to group leader property */
-      
+
       app = wnck_application_get (window->priv->group_leader);
-      
+
       if (app != NULL)
         return wnck_application_get_startup_id (app);
       else
@@ -1401,7 +1417,7 @@ wnck_window_is_sticky                 (WnckWindow *window)
  *
  * This function existed before 2.6, but the @timestamp argument was missing
  * in earlier versions.
- * 
+ *
  * Since: 2.6
  **/
 void
@@ -1425,7 +1441,8 @@ wnck_window_minimize                (WnckWindow *window)
 {
   g_return_if_fail (WNCK_IS_WINDOW (window));
 
-  _wnck_iconify (window->priv->xwindow);
+  _wnck_iconify (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                 window->priv->xwindow);
 }
 
 /**
@@ -1749,9 +1766,9 @@ wnck_window_keyboard_size (WnckWindow *window)
  * Gets the current workspace @window is on. If the window is pinned (on all
  * workspaces), or not on any workspaces, %NULL may be returned.
  *
- * Return value: the single current workspace @window is on, or %NULL. The
- * returned #WnckWorkspace is owned by libwnck and must not be referenced or
- * unreferenced.
+ * Return value: (transfer none): the single current workspace @window is on, or
+ * %NULL. The returned #WnckWorkspace is owned by libwnck and must not be
+ * referenced or unreferenced.
  **/
 WnckWorkspace*
 wnck_window_get_workspace (WnckWindow *window)
@@ -1769,9 +1786,8 @@ wnck_window_get_workspace (WnckWindow *window)
  * @window: a #WnckWindow.
  * @space: a #WnckWorkspace.
  *
- * Asks the window manager to move @window to @space.
- *
- * FIXME: what happens if @window is pinned?
+ * Asks the window manager to move @window to @space. If @window was pinned, it
+ * will also result in @window being visible only on @space.
  **/
 void
 wnck_window_move_to_workspace (WnckWindow    *window,
@@ -1859,7 +1875,7 @@ wnck_window_unpin (WnckWindow *window)
  *
  * This function existed before 2.10, but the @timestamp argument was missing
  * in earlier versions.
- * 
+ *
  * Since: 2.10
  **/
 void
@@ -1931,13 +1947,13 @@ static WnckWindow*
 find_last_transient_for (GList *windows,
                          Window xwindow)
 {
-  GList *tmp; 
+  GList *tmp;
   WnckWindow *retval;
 
   /* find _last_ transient for xwindow in the list */
-  
+
   retval = NULL;
-  
+
   tmp = windows;
   while (tmp != NULL)
     {
@@ -1946,7 +1962,7 @@ find_last_transient_for (GList *windows,
       if (w->priv->transient_for == xwindow &&
 	  w->priv->wintype != WNCK_WINDOW_UTILITY)
         retval = w;
-      
+
       tmp = tmp->next;
     }
 
@@ -1961,14 +1977,14 @@ find_last_transient_for (GList *windows,
  *
  * If @window has transients, activates the most likely transient
  * instead of the window itself. Otherwise activates @window.
- * 
+ *
  * FIXME the ideal behavior of this function is probably to activate
  * the most recently active window among @window and its transients.
  * This is probably best implemented on the window manager side.
- * 
+ *
  * This function existed before 2.10, but the @timestamp argument was missing
  * in earlier versions.
- * 
+ *
  * Since: 2.10
  **/
 void
@@ -1978,14 +1994,14 @@ wnck_window_activate_transient (WnckWindow *window,
   GList *windows;
   WnckWindow *transient;
   WnckWindow *next;
-  
+
   g_return_if_fail (WNCK_IS_WINDOW (window));
 
   windows = wnck_screen_get_windows_stacked (window->priv->screen);
 
   transient = NULL;
   next = find_last_transient_for (windows, window->priv->xwindow);
-  
+
   while (next != NULL)
     {
       if (next == window)
@@ -1996,7 +2012,7 @@ wnck_window_activate_transient (WnckWindow *window,
         }
 
       transient = next;
-      
+
       next = find_last_transient_for (windows, transient->priv->xwindow);
     }
 
@@ -2019,8 +2035,8 @@ wnck_window_activate_transient (WnckWindow *window,
  * (wnck_window_activate_transient), and clicking again should
  * minimize @window and its transients.  (Not doing this can be
  * especially annoying in the case of modal dialogs that don't appear
- * in the #WnckTaslist).
- * 
+ * in the #WnckTasklist).
+ *
  * Return value: %TRUE if one of the transients of @window is the most recently
  * activated window, %FALSE otherwise.
  *
@@ -2031,7 +2047,7 @@ wnck_window_transient_is_most_recently_activated (WnckWindow *window)
 {
   GList *windows;
   WnckWindow *transient;
-  
+
   g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
 
   windows = wnck_screen_get_windows_stacked (window->priv->screen);
@@ -2058,8 +2074,9 @@ get_icons (WnckWindow *window)
 
   icon = NULL;
   mini_icon = NULL;
-  
-  if (_wnck_read_icons (window->priv->xwindow,
+
+  if (_wnck_read_icons (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                        window->priv->xwindow,
                         window->priv->icon_cache,
                         &icon,
                         DEFAULT_ICON_WIDTH, DEFAULT_ICON_HEIGHT,
@@ -2086,14 +2103,14 @@ get_icons (WnckWindow *window)
 /**
  * wnck_window_get_icon:
  * @window: a #WnckWindow.
- * 
+ *
  * Gets the icon to be used for @window. If no icon was found, a fallback
  * icon is used. wnck_window_get_icon_is_fallback() can be used to tell if the
- * icon is the fallback icon. 
- * 
- * Return value: the icon for @window. The caller should reference the
- * returned <classname>GdkPixbuf</classname> if it needs to keep the icon
- * around.
+ * icon is the fallback icon.
+ *
+ * Return value: (transfer none): the icon for @window. The caller should
+ * reference the returned <classname>GdkPixbuf</classname> if it needs to keep
+ * the icon around.
  **/
 GdkPixbuf*
 wnck_window_get_icon (WnckWindow *window)
@@ -2112,26 +2129,26 @@ wnck_window_get_icon (WnckWindow *window)
 /**
  * wnck_window_get_mini_icon:
  * @window: a #WnckWindow.
- * 
+ *
  * Gets the mini-icon to be used for @window. If no mini-icon was found, a
  * fallback mini-icon is used. wnck_window_get_icon_is_fallback() can be used
- * to tell if the mini-icon is the fallback mini-icon. 
- * 
- * Return value: the mini-icon for @window. The caller should reference the
- * returned <classname>GdkPixbuf</classname> if it needs to keep the icon
- * around.
+ * to tell if the mini-icon is the fallback mini-icon.
+ *
+ * Return value: (transfer none): the mini-icon for @window. The caller should
+ * reference the returned <classname>GdkPixbuf</classname> if it needs to keep
+ * the icon around.
  **/
 GdkPixbuf*
 wnck_window_get_mini_icon (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
-  
+
   get_icons (window);
   if (window->priv->need_emit_icon_changed)
     queue_update (window); /* not done in get_icons since we call that from
                             * the update
                             */
-  
+
   return window->priv->mini_icon;
 }
 
@@ -2141,7 +2158,7 @@ wnck_window_get_mini_icon (WnckWindow *window)
  *
  * Gets whether a default fallback icon is used for @window (because none
  * was set on @window).
- * 
+ *
  * Return value: %TRUE if the icon for @window is a fallback, %FALSE otherwise.
  **/
 gboolean
@@ -2155,9 +2172,9 @@ wnck_window_get_icon_is_fallback (WnckWindow *window)
 /**
  * wnck_window_get_actions:
  * @window: a #WnckWindow.
- * 
+ *
  * Gets the actions that can be done for @window.
- * 
+ *
  * Return value: bitmask of actions that can be done for @window.
  **/
 WnckWindowActions
@@ -2172,9 +2189,9 @@ wnck_window_get_actions (WnckWindow *window)
 /**
  * wnck_window_get_state:
  * @window: a #WnckWindow.
- * 
+ *
  * Gets the state of @window.
- * 
+ *
  * Return value: bitmask of active states for @window.
  **/
 WnckWindowState
@@ -2300,7 +2317,7 @@ wnck_window_set_geometry (WnckWindow               *window,
   gravity_and_flags = gravity;
   gravity_and_flags |= geometry_mask << 8;
   gravity_and_flags |= source << 12;
-  
+
   x += window->priv->left_frame;
   y += window->priv->top_frame;
   width -= window->priv->left_frame + window->priv->right_frame;
@@ -2315,10 +2332,10 @@ wnck_window_set_geometry (WnckWindow               *window,
  * wnck_window_is_visible_on_workspace:
  * @window: a #WnckWindow.
  * @workspace: a #WnckWorkspace.
- * 
+ *
  * Like wnck_window_is_on_workspace(), but also checks that
  * the window is in a visible state (i.e. not minimized or shaded).
- * 
+ *
  * Return value: %TRUE if @window appears on @workspace in normal state, %FALSE
  * otherwise.
  **/
@@ -2330,7 +2347,7 @@ wnck_window_is_visible_on_workspace (WnckWindow    *window,
 
   g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
   g_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
-  
+
   state = wnck_window_get_state (window);
 
   if (state & WNCK_WINDOW_STATE_HIDDEN)
@@ -2370,7 +2387,8 @@ wnck_window_set_icon_geometry (WnckWindow *window,
   window->priv->icon_geometry.width = width;
   window->priv->icon_geometry.height = height;
 
-  _wnck_set_icon_geometry (window->priv->xwindow,
+  _wnck_set_icon_geometry (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                           window->priv->xwindow,
                            x, y, width, height);
 }
 
@@ -2378,7 +2396,7 @@ wnck_window_set_icon_geometry (WnckWindow *window,
  * wnck_window_is_on_workspace:
  * @window: a #WnckWindow.
  * @workspace: a #WnckWorkspace.
- * 
+ *
  * Gets whether @window appears on @workspace.
  *
  * Return value: %TRUE if @window appears on @workspace, %FALSE otherwise.
@@ -2389,7 +2407,7 @@ wnck_window_is_on_workspace (WnckWindow    *window,
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
   g_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
-  
+
   return wnck_window_is_pinned (window) ||
     wnck_window_get_workspace (window) == workspace;
 }
@@ -2398,9 +2416,9 @@ wnck_window_is_on_workspace (WnckWindow    *window,
  * wnck_window_is_in_viewport:
  * @window: a #WnckWindow.
  * @workspace: a #WnckWorkspace.
- * 
+ *
  * Gets %TRUE if @window appears in the current viewport of @workspace.
- * 
+ *
  * Return value: %TRUE if @window appears in current viewport of @workspace,
  * %FALSE otherwise.
  *
@@ -2412,7 +2430,7 @@ wnck_window_is_in_viewport (WnckWindow    *window,
 {
   GdkRectangle window_rect;
   GdkRectangle viewport_rect;
-  
+
   g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
   g_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
 
@@ -2593,7 +2611,8 @@ update_wm_state (WnckWindow *window)
 
   window->priv->wm_state_iconic = FALSE;
 
-  state = _wnck_get_wm_state (window->priv->xwindow);
+  state = _wnck_get_wm_state (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                              window->priv->xwindow);
 
   if (state == IconicState)
     window->priv->wm_state_iconic = TRUE;
@@ -2617,7 +2636,7 @@ update_state (WnckWindow *window)
    * state update just means the _NET_WM_STATE prop
    * changed
    */
-  
+
   if (reread_net_wm_state)
     {
       gboolean demanded_attention;
@@ -2635,10 +2654,11 @@ update_state (WnckWindow *window)
       window->priv->net_wm_state_hidden = FALSE;
       window->priv->is_fullscreen = FALSE;
       window->priv->demands_attention = FALSE;
-      
+
       atoms = NULL;
       n_atoms = 0;
-      _wnck_get_atom_list (window->priv->xwindow,
+      _wnck_get_atom_list (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                           window->priv->xwindow,
                            _wnck_atom_get ("_NET_WM_STATE"),
                            &atoms, &n_atoms);
 
@@ -2701,7 +2721,7 @@ update_state (WnckWindow *window)
           !window->priv->transient_for_root)
         window->priv->skip_taskbar = TRUE;
       break;
-      
+
     case WNCK_WINDOW_NORMAL:
       break;
     }
@@ -2718,13 +2738,13 @@ update_state (WnckWindow *window)
     case WNCK_WINDOW_SPLASHSCREEN:
       window->priv->skip_pager = TRUE;
       break;
-      
+
     case WNCK_WINDOW_NORMAL:
     case WNCK_WINDOW_DIALOG:
     case WNCK_WINDOW_UTILITY:
       break;
     }
-  
+
   /* FIXME we need to recompute this if the window manager changes */
   if (wnck_screen_net_wm_supports (window->priv->screen,
                                    "_NET_WM_STATE_HIDDEN"))
@@ -2740,20 +2760,11 @@ update_state (WnckWindow *window)
   else
     {
       window->priv->is_minimized = window->priv->wm_state_iconic;
-      
+
       window->priv->is_hidden = window->priv->is_minimized || window->priv->is_shaded;
     }
 }
 
-static gboolean
-nullstr_equal (const char *str1, const char *str2)
-{
-  if (str1 == NULL || str2 == NULL)
-    return str1 == str2;
-  else
-    return !strcmp (str1, str2);
-}
-
 static void
 update_name (WnckWindow *window)
 {
@@ -2764,9 +2775,10 @@ update_name (WnckWindow *window)
 
   window->priv->need_update_name = FALSE;
 
-  new_name = _wnck_get_name (window->priv->xwindow);
+  new_name = _wnck_get_name (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                             window->priv->xwindow);
 
-  if (!nullstr_equal (window->priv->name, new_name))
+  if (g_strcmp0 (window->priv->name, new_name) != 0)
     window->priv->need_emit_name_changed = TRUE;
 
   g_free (window->priv->name);
@@ -2783,9 +2795,10 @@ update_icon_name (WnckWindow *window)
 
   window->priv->need_update_icon_name = FALSE;
 
-  new_name = _wnck_get_icon_name (window->priv->xwindow);
+  new_name = _wnck_get_icon_name (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                                  window->priv->xwindow);
 
-  if (!nullstr_equal (window->priv->icon_name, new_name))
+  if (g_strcmp0 (window->priv->icon_name, new_name) != 0)
     window->priv->need_emit_name_changed = TRUE;
 
   g_free (window->priv->icon_name);
@@ -2806,7 +2819,8 @@ update_workspace (WnckWindow *window)
   old = window->priv->workspace;
 
   val = ALL_WORKSPACES;
-  _wnck_get_cardinal (window->priv->xwindow,
+  _wnck_get_cardinal (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                      window->priv->xwindow,
                       _wnck_atom_get ("_NET_WM_DESKTOP"),
                       &val);
 
@@ -2832,12 +2846,13 @@ update_actions (WnckWindow *window)
 
   atoms = NULL;
   n_atoms = 0;
-  if (!_wnck_get_atom_list (window->priv->xwindow,
+  if (!_wnck_get_atom_list (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                            window->priv->xwindow,
                             _wnck_atom_get ("_NET_WM_ALLOWED_ACTIONS"),
                             &atoms,
                             &n_atoms))
     {
-      window->priv->actions = 
+      window->priv->actions =
                 WNCK_WINDOW_ACTION_MOVE                    |
                 WNCK_WINDOW_ACTION_RESIZE                  |
                 WNCK_WINDOW_ACTION_SHADE                   |
@@ -2929,7 +2944,7 @@ update_wintype (WnckWindow *window)
   int n_atoms;
   WnckWindowType type;
   gboolean found_type;
-  
+
   if (!window->priv->need_update_wintype)
     return;
 
@@ -2937,16 +2952,17 @@ update_wintype (WnckWindow *window)
 
   found_type = FALSE;
   type = WNCK_WINDOW_NORMAL;
-  
+
   atoms = NULL;
   n_atoms = 0;
-  if (_wnck_get_atom_list (window->priv->xwindow,
+  if (_wnck_get_atom_list (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                           window->priv->xwindow,
                            _wnck_atom_get ("_NET_WM_WINDOW_TYPE"),
                            &atoms,
                            &n_atoms))
     {
       int i;
-      
+
       i = 0;
       while (i < n_atoms && !found_type)
         {
@@ -2972,7 +2988,7 @@ update_wintype (WnckWindow *window)
             type = WNCK_WINDOW_SPLASHSCREEN;
           else
             found_type = FALSE;
-          
+
           ++i;
         }
 
@@ -3004,9 +3020,10 @@ update_transient_for (WnckWindow *window)
     return;
 
   window->priv->need_update_transient_for = FALSE;
-  
+
   parent = None;
-  if (_wnck_get_window (window->priv->xwindow,
+  if (_wnck_get_window (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                        window->priv->xwindow,
                         _wnck_atom_get ("WM_TRANSIENT_FOR"),
                         &parent) &&
       parent != window->priv->xwindow)
@@ -3034,8 +3051,9 @@ update_startup_id (WnckWindow *window)
   window->priv->need_update_startup_id = FALSE;
 
   g_free (window->priv->startup_id);
-  window->priv->startup_id = 
-    _wnck_get_utf8_property (window->priv->xwindow,
+  window->priv->startup_id =
+    _wnck_get_utf8_property (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                             window->priv->xwindow,
                              _wnck_atom_get ("_NET_STARTUP_ID"));
 }
 
@@ -3053,7 +3071,8 @@ update_wmclass (WnckWindow *window)
   window->priv->res_class = NULL;
   window->priv->res_name = NULL;
 
-  _wnck_get_wmclass (window->priv->xwindow,
+  _wnck_get_wmclass (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                     window->priv->xwindow,
                      &window->priv->res_class,
                      &window->priv->res_name);
 }
@@ -3061,14 +3080,17 @@ update_wmclass (WnckWindow *window)
 static void
 update_wmhints (WnckWindow *window)
 {
+  Display  *display;
   XWMHints *hints;
 
   if (!window->priv->need_update_wmhints)
     return;
 
-  _wnck_error_trap_push ();
-  hints = XGetWMHints (gdk_display, window->priv->xwindow);
-  _wnck_error_trap_pop ();
+  display = _wnck_window_get_display (window);
+
+  _wnck_error_trap_push (display);
+  hints = XGetWMHints (display, window->priv->xwindow);
+  _wnck_error_trap_pop (display);
 
   if (hints)
     {
@@ -3110,7 +3132,8 @@ update_frame_extents (WnckWindow *window)
 
   left = right = top = bottom = 0;
 
-  if (!_wnck_get_frame_extents (window->priv->xwindow,
+  if (!_wnck_get_frame_extents (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                                window->priv->xwindow,
                                 &left, &right, &top, &bottom))
     return;
 
@@ -3134,7 +3157,7 @@ force_update_now (WnckWindow *window)
   WnckWindowState old_state;
   WnckWindowState new_state;
   WnckWindowActions old_actions;
-  
+
   unqueue_update (window);
 
   /* Name must be done before all other stuff,
@@ -3166,7 +3189,7 @@ force_update_now (WnckWindow *window)
   update_frame_extents (window); /* emits signals */
 
   get_icons (window);
-  
+
   new_state = COMPRESS_STATE (window);
 
   if (old_state != new_state)
@@ -3175,7 +3198,7 @@ force_update_now (WnckWindow *window)
   if (old_actions != window->priv->actions)
     emit_actions_changed (window, old_actions ^ window->priv->actions,
                           window->priv->actions);
-  
+
   if (window->priv->need_emit_icon_changed)
     emit_icon_changed (window);
 }
